@@ -1,3 +1,4 @@
+# Copyright (c) OpenMMLab. All rights reserved.
 import math
 import os
 import shutil
@@ -46,7 +47,7 @@ def show_feature(features, names, to_uint8, out_file=None):
         out_file (str): The output file name. If set to None,
             the output image will be shown without saving.
     """
-    assert utils.is_ndarray_list(features)
+    assert utils.is_type_list(features, np.ndarray)
     assert utils.is_type_list(names, str)
     assert utils.is_type_list(to_uint8, int)
     assert utils.is_none_or_type(out_file, str)
@@ -75,8 +76,7 @@ def show_img_boundary(img, boundary):
         boundary (list[float or int]): The input boundary.
     """
     assert isinstance(img, np.ndarray)
-    assert utils.is_type_list(boundary, int) or utils.is_type_list(
-        boundary, float)
+    assert utils.is_type_list(boundary, (int, float))
 
     cv2.polylines(
         img, [np.array(boundary).astype(np.int32).reshape(-1, 1, 2)],
@@ -380,23 +380,23 @@ def imshow_text_label(img,
     return img
 
 
-def imshow_edge_node(img,
-                     result,
-                     boxes,
-                     idx_to_cls={},
-                     show=False,
-                     win_name='',
-                     wait_time=-1,
-                     out_file=None):
+def imshow_node(img,
+                result,
+                boxes,
+                idx_to_cls={},
+                show=False,
+                win_name='',
+                wait_time=-1,
+                out_file=None):
 
     img = mmcv.imread(img)
     h, w = img.shape[:2]
 
-    pred_img = np.ones((h, w * 2, 3), dtype=np.uint8) * 255
     max_value, max_idx = torch.max(result['nodes'].detach().cpu(), -1)
     node_pred_label = max_idx.numpy().tolist()
     node_pred_score = max_value.numpy().tolist()
 
+    texts, text_boxes = [], []
     for i, box in enumerate(boxes):
         new_box = [[box[0], box[1]], [box[2], box[1]], [box[2], box[3]],
                    [box[0], box[3]]]
@@ -409,13 +409,30 @@ def imshow_edge_node(img,
         x_min = int(min([point[0] for point in new_box]))
         y_min = int(min([point[1] for point in new_box]))
 
+        # text
         pred_label = str(node_pred_label[i])
         if pred_label in idx_to_cls:
             pred_label = idx_to_cls[pred_label]
         pred_score = '{:.2f}'.format(node_pred_score[i])
         text = pred_label + '(' + pred_score + ')'
-        cv2.putText(pred_img, text, (x_min * 2, y_min),
-                    cv2.FONT_HERSHEY_SIMPLEX, 0.5, (255, 0, 0), 1)
+        texts.append(text)
+
+        # text box
+        font_size = int(
+            min(
+                abs(new_box[3][1] - new_box[0][1]),
+                abs(new_box[1][0] - new_box[0][0])))
+        char_num = len(text)
+        text_box = [
+            x_min * 2, y_min, x_min * 2 + font_size * char_num, y_min,
+            x_min * 2 + font_size * char_num, y_min + font_size, x_min * 2,
+            y_min + font_size
+        ]
+        text_boxes.append(text_box)
+
+    pred_img = np.ones((h, w * 2, 3), dtype=np.uint8) * 255
+    pred_img = draw_texts_by_pil(
+        pred_img, texts, text_boxes, draw_box=False, on_ori_img=True)
 
     vis_img = np.ones((h, w * 3, 3), dtype=np.uint8) * 255
     vis_img[:, :w] = img
@@ -481,27 +498,38 @@ def get_optimal_font_scale(text, width):
     return 1
 
 
-def draw_texts(img, boxes, texts):
+def draw_texts(img, texts, boxes=None, draw_box=True, on_ori_img=False):
     """Draw boxes and texts on empty img.
 
     Args:
         img (np.ndarray): The original image.
-        boxes (list[list[float]]): Detected bounding boxes.
         texts (list[str]): Recognized texts.
+        boxes (list[list[float]]): Detected bounding boxes.
+        draw_box (bool): Whether draw box or not. If False, draw text only.
+        on_ori_img (bool): If True, draw box and text on input image,
+            else, on a new empty image.
     Return:
         out_img (np.ndarray): Visualized image.
     """
     color_list = gen_color()
     h, w = img.shape[:2]
-    out_img = np.ones((h, w, 3), dtype=np.uint8) * 255
+    if boxes is None:
+        boxes = [[0, 0, w, 0, w, h, 0, h]]
+    assert len(texts) == len(boxes)
+
+    if on_ori_img:
+        out_img = img
+    else:
+        out_img = np.ones((h, w, 3), dtype=np.uint8) * 255
     for idx, (box, text) in enumerate(zip(boxes, texts)):
-        new_box = [[x, y] for x, y in zip(box[0::2], box[1::2])]
-        Pts = np.array([new_box], np.int32)
-        cv2.polylines(
-            out_img, [Pts.reshape((-1, 1, 2))],
-            True,
-            color=color_list[idx % len(color_list)],
-            thickness=1)
+        if draw_box:
+            new_box = [[x, y] for x, y in zip(box[0::2], box[1::2])]
+            Pts = np.array([new_box], np.int32)
+            cv2.polylines(
+                out_img, [Pts.reshape((-1, 1, 2))],
+                True,
+                color=color_list[idx % len(color_list)],
+                thickness=1)
         min_x = int(min(box[0::2]))
         max_y = int(
             np.mean(np.array(box[1::2])) + 0.2 *
@@ -514,31 +542,63 @@ def draw_texts(img, boxes, texts):
     return out_img
 
 
-def draw_texts_by_pil(img, texts, boxes=None):
+def draw_texts_by_pil(img,
+                      texts,
+                      boxes=None,
+                      draw_box=True,
+                      on_ori_img=False,
+                      font_size=None,
+                      fill_color=None,
+                      draw_pos=None,
+                      return_text_size=False):
     """Draw boxes and texts on empty image, especially for Chinese.
 
     Args:
         img (np.ndarray): The original image.
         texts (list[str]): Recognized texts.
         boxes (list[list[float]]): Detected bounding boxes.
-    Return:
-        out_img (np.ndarray): Visualized text image.
+        draw_box (bool): Whether draw box or not. If False, draw text only.
+        on_ori_img (bool): If True, draw box and text on input image,
+            else on a new empty image.
+        font_size (int, optional): Size to create a font object for a font.
+        fill_color (tuple(int), optional): Fill color for text.
+        draw_pos (list[tuple(int)], optional): Start point to draw each text.
+        return_text_size (bool): If True, return the list of text size.
+
+    Returns:
+        (np.ndarray, list[tuple]) or np.ndarray: Return a tuple
+        ``(out_img, text_sizes)``, where ``out_img`` is the output image
+        with texts drawn on it and ``text_sizes`` are the size of drawing
+        texts. If ``return_text_size`` is False, only the output image will be
+        returned.
     """
 
     color_list = gen_color()
     h, w = img.shape[:2]
     if boxes is None:
         boxes = [[0, 0, w, 0, w, h, 0, h]]
+    if draw_pos is None:
+        draw_pos = [None for _ in texts]
+    assert len(boxes) == len(texts) == len(draw_pos)
 
-    out_img = Image.new('RGB', (w, h), color=(255, 255, 255))
+    if fill_color is None:
+        fill_color = (0, 0, 0)
+
+    if on_ori_img:
+        out_img = Image.fromarray(cv2.cvtColor(img, cv2.COLOR_BGR2RGB))
+    else:
+        out_img = Image.new('RGB', (w, h), color=(255, 255, 255))
     out_draw = ImageDraw.Draw(out_img)
-    for idx, (box, text) in enumerate(zip(boxes, texts)):
+
+    text_sizes = []
+    for idx, (box, text, ori_point) in enumerate(zip(boxes, texts, draw_pos)):
+        if len(text) == 0:
+            continue
         min_x, max_x = min(box[0::2]), max(box[0::2])
         min_y, max_y = min(box[1::2]), max(box[1::2])
         color = tuple(list(color_list[idx % len(color_list)])[::-1])
-        out_draw.line(box, fill=color, width=1)
-        box_width = max(max_x - min_x, max_y - min_y)
-        font_size = int(0.9 * box_width / len(text))
+        if draw_box:
+            out_draw.line(box, fill=color, width=1)
         dirname, _ = os.path.split(os.path.abspath(__file__))
         font_path = os.path.join(dirname, 'font.TTF')
         if not os.path.exists(font_path):
@@ -546,13 +606,23 @@ def draw_texts_by_pil(img, texts, boxes=None):
             print(f'Downloading {url} ...')
             local_filename, _ = urllib.request.urlretrieve(url)
             shutil.move(local_filename, font_path)
-        fnt = ImageFont.truetype(font_path, font_size)
-        out_draw.text((min_x + 1, min_y + 1), text, font=fnt, fill=(0, 0, 0))
+        tmp_font_size = font_size
+        if tmp_font_size is None:
+            box_width = max(max_x - min_x, max_y - min_y)
+            tmp_font_size = int(0.9 * box_width / len(text))
+        fnt = ImageFont.truetype(font_path, tmp_font_size)
+        if ori_point is None:
+            ori_point = (min_x + 1, min_y + 1)
+        out_draw.text(ori_point, text, font=fnt, fill=fill_color)
+        text_sizes.append(fnt.getsize(text))
 
     del out_draw
 
     out_img = cv2.cvtColor(np.asarray(out_img), cv2.COLOR_RGB2BGR)
 
+    if return_text_size:
+        return out_img, text_sizes
+
     return out_img
 
 
@@ -570,12 +640,13 @@ def is_contain_chinese(check_str):
     return False
 
 
-def det_recog_show_result(img, end2end_res):
+def det_recog_show_result(img, end2end_res, out_file=None):
     """Draw `result`(boxes and texts) on `img`.
+
     Args:
         img (str or np.ndarray): The image to be displayed.
         end2end_res (dict): Text detect and recognize results.
-
+        out_file (str): Image path where the visualized image should be saved.
     Return:
         out_img (np.ndarray): Visualized image.
     """
@@ -589,11 +660,213 @@ def det_recog_show_result(img, end2end_res):
     if is_contain_chinese(''.join(texts)):
         text_vis_img = draw_texts_by_pil(img, texts, boxes)
     else:
-        text_vis_img = draw_texts(img, boxes, texts)
+        text_vis_img = draw_texts(img, texts, boxes)
 
     h, w = img.shape[:2]
     out_img = np.ones((h, w * 2, 3), dtype=np.uint8)
     out_img[:, :w, :] = box_vis_img
     out_img[:, w:, :] = text_vis_img
 
+    if out_file:
+        mmcv.imwrite(out_img, out_file)
+
     return out_img
+
+
+def draw_edge_result(img, result, edge_thresh=0.5, keynode_thresh=0.5):
+    """Draw text and their relationship on empty images.
+
+    Args:
+        img (np.ndarray): The original image.
+        result (dict): The result of model forward_test, including:
+            - img_metas (list[dict]): List of meta information dictionary.
+            - nodes (Tensor): Node prediction with size:
+                number_node * node_classes.
+            - edges (Tensor): Edge prediction with size: number_edge * 2.
+        edge_thresh (float): Score threshold for edge classification.
+        keynode_thresh (float): Score threshold for node
+            (``key``) classification.
+
+    Returns:
+        np.ndarray: The image with key, value and relation drawn on it.
+    """
+
+    h, w = img.shape[:2]
+
+    vis_area_width = w // 3 * 2
+    vis_area_height = h
+    dist_key_to_value = vis_area_width // 2
+    dist_pair_to_pair = 30
+
+    bbox_x1 = dist_pair_to_pair
+    bbox_y1 = 0
+
+    new_w = vis_area_width
+    new_h = vis_area_height
+    pred_edge_img = np.ones((new_h, new_w, 3), dtype=np.uint8) * 255
+
+    nodes = result['nodes'].detach().cpu()
+    texts = result['img_metas'][0]['ori_texts']
+    num_nodes = result['nodes'].size(0)
+    edges = result['edges'].detach().cpu()[:, -1].view(num_nodes, num_nodes)
+
+    # (i, j) will be a valid pair
+    # either edge_score(node_i->node_j) > edge_thresh
+    # or edge_score(node_j->node_i) > edge_thresh
+    pairs = (torch.max(edges, edges.T) > edge_thresh).nonzero(as_tuple=True)
+
+    # 1. "for n1, n2 in zip(*pairs) if n1 < n2":
+    #     Only (n1, n2) will be included if n1 < n2 but not (n2, n1), to
+    #     avoid duplication.
+    # 2. "(n1, n2) if nodes[n1, 1] > nodes[n1, 2]":
+    #     nodes[n1, 1] is the score that this node is predicted as key,
+    #     nodes[n1, 2] is the score that this node is predicted as value.
+    #     If nodes[n1, 1] > nodes[n1, 2], n1 will be the index of key,
+    #     so that n2 will be the index of value.
+    result_pairs = [(n1, n2) if nodes[n1, 1] > nodes[n1, 2] else (n2, n1)
+                    for n1, n2 in zip(*pairs) if n1 < n2]
+
+    result_pairs.sort()
+    key_current_idx = -1
+    pos_current = (-1, -1)
+    newline_flag = False
+
+    key_font_size = 15
+    value_font_size = 15
+    key_font_color = (0, 0, 0)
+    value_font_color = (0, 0, 255)
+    arrow_color = (0, 0, 255)
+    for pair in result_pairs:
+        key_idx = int(pair[0].item())
+        if nodes[key_idx, 1] < keynode_thresh:
+            continue
+        if key_idx != key_current_idx:
+            # move y-coords down for a new key
+            bbox_y1 += 10
+            # enlarge blank area to show key-value info
+            if newline_flag:
+                bbox_x1 += vis_area_width
+                tmp_img = np.ones(
+                    (new_h, new_w + vis_area_width, 3), dtype=np.uint8) * 255
+                tmp_img[:new_h, :new_w] = pred_edge_img
+                pred_edge_img = tmp_img
+                new_w += vis_area_width
+                newline_flag = False
+                bbox_y1 = 10
+        key_text = texts[key_idx]
+        key_pos = (bbox_x1, bbox_y1)
+        value_idx = pair[1].item()
+        value_text = texts[value_idx]
+        value_pos = (bbox_x1 + dist_key_to_value, bbox_y1)
+        if key_idx != key_current_idx:
+            # draw text for a new key
+            key_current_idx = key_idx
+            pred_edge_img, text_sizes = draw_texts_by_pil(
+                pred_edge_img, [key_text],
+                draw_box=False,
+                on_ori_img=True,
+                font_size=key_font_size,
+                fill_color=key_font_color,
+                draw_pos=[key_pos],
+                return_text_size=True)
+            pos_right_bottom = (key_pos[0] + text_sizes[0][0],
+                                key_pos[1] + text_sizes[0][1])
+            pos_current = (pos_right_bottom[0] + 5, bbox_y1 + 10)
+            pred_edge_img = cv2.arrowedLine(
+                pred_edge_img, (pos_right_bottom[0] + 5, bbox_y1 + 10),
+                (bbox_x1 + dist_key_to_value - 5, bbox_y1 + 10), arrow_color,
+                1)
+        else:
+            # draw arrow from key to value
+            if newline_flag:
+                tmp_img = np.ones((new_h + dist_pair_to_pair, new_w, 3),
+                                  dtype=np.uint8) * 255
+                tmp_img[:new_h, :new_w] = pred_edge_img
+                pred_edge_img = tmp_img
+                new_h += dist_pair_to_pair
+            pred_edge_img = cv2.arrowedLine(pred_edge_img, pos_current,
+                                            (bbox_x1 + dist_key_to_value - 5,
+                                             bbox_y1 + 10), arrow_color, 1)
+        # draw text for value
+        pred_edge_img = draw_texts_by_pil(
+            pred_edge_img, [value_text],
+            draw_box=False,
+            on_ori_img=True,
+            font_size=value_font_size,
+            fill_color=value_font_color,
+            draw_pos=[value_pos],
+            return_text_size=False)
+        bbox_y1 += dist_pair_to_pair
+        if bbox_y1 + dist_pair_to_pair >= new_h:
+            newline_flag = True
+
+    return pred_edge_img
+
+
+def imshow_edge(img,
+                result,
+                boxes,
+                show=False,
+                win_name='',
+                wait_time=-1,
+                out_file=None):
+    """Display the prediction results of the nodes and edges of the KIE model.
+
+    Args:
+        img (np.ndarray): The original image.
+        result (dict): The result of model forward_test, including:
+            - img_metas (list[dict]): List of meta information dictionary.
+            - nodes (Tensor): Node prediction with size: \
+                number_node * node_classes.
+            - edges (Tensor): Edge prediction with size: number_edge * 2.
+        boxes (list): The text boxes corresponding to the nodes.
+        show (bool): Whether to show the image. Default: False.
+        win_name (str): The window name. Default: ''
+        wait_time (float): Value of waitKey param. Default: 0.
+        out_file (str or None): The filename to write the image.
+            Default: None.
+
+    Returns:
+        np.ndarray: The image with key, value and relation drawn on it.
+    """
+    img = mmcv.imread(img)
+    h, w = img.shape[:2]
+    color_list = gen_color()
+
+    for i, box in enumerate(boxes):
+        new_box = [[box[0], box[1]], [box[2], box[1]], [box[2], box[3]],
+                   [box[0], box[3]]]
+        Pts = np.array([new_box], np.int32)
+        cv2.polylines(
+            img, [Pts.reshape((-1, 1, 2))],
+            True,
+            color=color_list[i % len(color_list)],
+            thickness=1)
+
+    pred_img_h = h
+    pred_img_w = w
+
+    pred_edge_img = draw_edge_result(img, result)
+    pred_img_h = max(pred_img_h, pred_edge_img.shape[0])
+    pred_img_w += pred_edge_img.shape[1]
+
+    vis_img = np.zeros((pred_img_h, pred_img_w, 3), dtype=np.uint8)
+    vis_img[:h, :w] = img
+    vis_img[:, w:] = 255
+
+    height_t, width_t = pred_edge_img.shape[:2]
+    vis_img[:height_t, w:(w + width_t)] = pred_edge_img
+
+    if show:
+        mmcv.imshow(vis_img, win_name, wait_time)
+    if out_file is not None:
+        mmcv.imwrite(vis_img, out_file)
+        res_dic = {
+            'boxes': boxes,
+            'nodes': result['nodes'].detach().cpu(),
+            'edges': result['edges'].detach().cpu(),
+            'metas': result['img_metas'][0]
+        }
+        mmcv.dump(res_dic, f'{out_file}_res.pkl')
+
+    return vis_img
