@@ -1,5 +1,6 @@
-from . import (backbones, convertors, decoders, encoders, heads, losses, necks,
-               preprocessor, recognizer)
+# Copyright (c) OpenMMLab. All rights reserved.
+from . import (backbones, convertors, decoders, encoders, fusers, heads,
+               losses, necks, preprocessor, recognizer)
 
 from .backbones import *  # NOQA
 from .convertors import *  # NOQA
@@ -10,8 +11,9 @@ from .losses import *  # NOQA
 from .necks import *  # NOQA
 from .preprocessor import *  # NOQA
 from .recognizer import *  # NOQA
+from .fusers import *  # NOQA
 
 __all__ = (
     backbones.__all__ + convertors.__all__ + decoders.__all__ +
     encoders.__all__ + heads.__all__ + losses.__all__ + necks.__all__ +
-    preprocessor.__all__ + recognizer.__all__)
+    preprocessor.__all__ + recognizer.__all__ + fusers.__all__)
